@@ -2,7 +2,7 @@
 #include <stdlib.h>
 #include "eog-file-selection.h"
 #include <libgnome/gnome-macros.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 #include <glib/gslist.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenuitem.h>
@@ -13,8 +13,11 @@
 #include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkvbox.h>
+#include <gtk/gtkcheckbutton.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomeui/gnome-thumbnail.h>
+#include <gconf/gconf-client.h>
+#include "eog-config-keys.h"
 #include "eog-hig-dialog.h"
 #include "eog-pixbuf-util.h"
 
@@ -83,67 +86,16 @@ eog_file_selection_instance_init (EogFileSelection *filesel)
 	filesel->priv = priv;
 }
 
-static gboolean
-replace_file (GtkWindow *window, const gchar *file_name)
-{
-       GtkWidget *msgbox;
-       gchar *utf_file_name;
-       gchar *msg;
-       gint ret;
-
-       utf_file_name = g_filename_to_utf8 (file_name, -1,
-                                            NULL, NULL, NULL);
-       msg = g_strdup_printf (_("Do you want to overwrite %s?"),
-			      utf_file_name);
-       g_free (utf_file_name);
-       
-       msgbox = eog_hig_dialog_new (GTK_STOCK_DIALOG_WARNING,
-                                    _("File exists"), 
-                                    msg,
-                                    TRUE);
-
-       gtk_dialog_add_buttons (GTK_DIALOG (msgbox),
-                               GTK_STOCK_NO,
-                               GTK_RESPONSE_CANCEL,
-                               GTK_STOCK_YES,
-                               GTK_RESPONSE_YES,
-                               NULL);
-
-       gtk_dialog_set_default_response (GTK_DIALOG (msgbox),
-                                        GTK_RESPONSE_CANCEL);
-
-       ret = gtk_dialog_run (GTK_DIALOG (msgbox));
-       gtk_widget_destroy (msgbox);
-       g_free (msg);
-
-       return (ret == GTK_RESPONSE_YES);
-}
-
 static void
 response_cb (GtkDialog *dlg, gint id, gpointer data)
 {
 	char *dir;
 	GtkFileChooserAction action;
-		
-	dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg));
-	action = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (dlg));
-	
-        if (action == GTK_FILE_CHOOSER_ACTION_SAVE && id == GTK_RESPONSE_OK){
-		char *filename;
-		filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dlg));
 
-		if (g_file_test (filename, G_FILE_TEST_EXISTS) &&
-		    !replace_file (GTK_WINDOW (dlg), filename)) 
-		{
-			g_signal_stop_emission_by_name (G_OBJECT (dlg), "response");
-			g_free (filename);
-			return;
-		}
-		
-		g_free (filename);
-	}
-	
 	if (id == GTK_RESPONSE_OK) {
+		dir = gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (dlg));
+		action = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (dlg));
+		
 		if (last_dir [action] != NULL)
 			g_free (last_dir [action]);
 		
@@ -189,15 +141,19 @@ eog_file_selection_add_filter (EogFileSelection *filesel)
 	/* Image filters */
 	for (it = formats; it != NULL; it = it->next) {
 		char *filter_name;
+		char *description, *extension;
 		GdkPixbufFormat *format;
 		filter = gtk_file_filter_new ();
 
 		format = (GdkPixbufFormat*) it->data;
+		description = gdk_pixbuf_format_get_description (format);
+		extension = gdk_pixbuf_format_get_name (format);
 
 		/* Filter name: First description then file extension, eg. "The PNG-Format (*.png)".*/
-		filter_name = g_strdup_printf (_("%s (*.%s)"), 
-					       gdk_pixbuf_format_get_description (format),
-					       gdk_pixbuf_format_get_name (format));
+		filter_name = g_strdup_printf (_("%s (*.%s)"), description, extension);
+		g_free (description);
+		g_free (extension);
+
 		gtk_file_filter_set_name (filter, filter_name);
 		g_free (filter_name);
 
@@ -258,6 +214,7 @@ set_preview_pixbuf (EogFileSelection *filesel, GdkPixbuf *pixbuf, GnomeVFSFileIn
 {
 	EogFileSelectionPrivate *priv;
 	int bytes;
+	int pixels;
 	const char *bytes_str;
 	const char *width;
 	const char *height;
@@ -287,8 +244,9 @@ set_preview_pixbuf (EogFileSelection *filesel, GdkPixbuf *pixbuf, GnomeVFSFileIn
 		height = gdk_pixbuf_get_option (pixbuf, "tEXt::Thumb::Image::Height");
 		
 		if ((width != NULL) && (height != NULL)) {
+			pixels = atoi (height);
 			/* Pixel size of image: width x height in pixel */
-			dim_str = g_strdup_printf (_("%s x %s pixel"), width, height);
+			dim_str = g_strdup_printf (ngettext ("%s x %s pixels", "%s x %s pixels", pixels), width, height);
 		}
 
 #if 0
@@ -428,6 +386,38 @@ eog_file_selection_add_preview (GtkWidget *widget)
 			  G_CALLBACK (update_preview_cb), NULL);
 }
 
+static void
+eog_file_selection_toggle_button_cb (GtkToggleButton *button, gpointer user_data)
+{
+	GConfClient *client;	
+
+	client = gconf_client_get_default ();
+	gconf_client_set_bool (client, EOG_CONF_WINDOW_OPEN_NEW_WINDOW,
+			       gtk_toggle_button_get_active (button), NULL);
+	g_object_unref (client);
+}
+
+static void
+eog_file_selection_add_open_new_window (GtkWidget *widget)
+{
+	GConfClient *client;
+	gboolean new_window;
+	GtkWidget *check_button;
+
+	client = gconf_client_get_default ();
+	new_window = gconf_client_get_bool (client, EOG_CONF_WINDOW_OPEN_NEW_WINDOW, NULL);
+	g_object_unref (client);
+
+	check_button = gtk_check_button_new_with_label (_("Open in new window"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_button), new_window);
+
+	g_signal_connect (G_OBJECT (check_button), "toggled", 
+			  (GCallback) eog_file_selection_toggle_button_cb, NULL);
+
+	gtk_box_pack_end (GTK_BOX (GTK_DIALOG (widget)->vbox), check_button, FALSE, FALSE, 0);
+	gtk_widget_show (check_button);
+}
+
 GtkWidget* 
 eog_file_selection_new (GtkFileChooserAction action)
 {
@@ -471,6 +461,9 @@ eog_file_selection_new (GtkFileChooserAction action)
 		eog_file_selection_add_filter (EOG_FILE_SELECTION (filesel));
 		eog_file_selection_add_preview (filesel);
 	}
+	eog_file_selection_add_open_new_window (filesel);
+
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (filesel), FALSE);
 
 	if (last_dir[action] != NULL) {
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (filesel), last_dir [action]);
@@ -499,4 +492,3 @@ eog_file_selection_get_format (EogFileSelection *sel)
 	
 	return format;
 }
-
