@@ -7,17 +7,6 @@
 #endif
 #include "eog-image.h"
 
-enum {
-	EOG_IMAGE_LOAD_STATUS_NONE     = 0,
-	EOG_IMAGE_LOAD_STATUS_PREPARED = 1 << 0,
-	EOG_IMAGE_LOAD_STATUS_UPDATED  = 1 << 1,
-	EOG_IMAGE_LOAD_STATUS_DONE     = 1 << 2,
-	EOG_IMAGE_LOAD_STATUS_FAILED   = 1 << 3,
-	EOG_IMAGE_LOAD_STATUS_CANCELLED = 1 << 4,
-	EOG_IMAGE_LOAD_STATUS_INFO_DONE = 1 << 5,
-	EOG_IMAGE_LOAD_STATUS_TRANSFORMED = 1 << 6,
-	EOG_IMAGE_LOAD_STATUS_PROGRESS = 1 << 7
-};
 
 typedef enum {
 	EOG_IMAGE_STATUS_UNKNOWN,
@@ -28,7 +17,6 @@ typedef enum {
  
 struct _EogImagePrivate {
 	GnomeVFSURI *uri;
-	EogImageLoadMode mode;
 	EogImageStatus status;
 
 	GdkPixbuf *image;
@@ -38,10 +26,17 @@ struct _EogImagePrivate {
 	gint height;
 	GnomeVFSFileSize bytes;
 	char *file_type;
-#if HAVE_EXIF
-	ExifData *exif;
-#endif
 
+	guchar  *exif_chunk; /* holds EXIF raw data */
+	guint    exif_chunk_len;
+	guchar  *iptc_chunk; /* holds IPTC raw data */
+	guint    iptc_chunk_len;
+#if HAVE_EXIF
+	ExifData *exif;      /* this is mutual exclusive to exif_chunk. Only 
+			      * either of these are not NULL:
+			      */
+#endif
+	
 	gint thumbnail_id;
 	
 	gboolean modified;
@@ -50,24 +45,20 @@ struct _EogImagePrivate {
 	gchar *caption_key;
 
 	GThread *load_thread;
-	gint load_id;
 	GMutex *status_mutex;
-	gint load_status;
+	GCond  *load_finished;
 	gboolean cancel_loading;
 	float progress; /* Range from [0.0...1.0] indicate the progress of 
 			   actions in percent */
 
-	/* data which depends on the load status */
-	int update_x1;
-	int update_y1;
-	int update_x2;
-	int update_y2;
 	char *error_message;
 	
 	/* stack of transformations recently applied */
 	GList *undo_stack;
 	/* composition of all applied transformations */
 	EogTransform *trans;
+
+	guint data_ref_count;
 };
 
 
