@@ -1,6 +1,11 @@
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <stdlib.h>
-#include <libgnome/libgnome.h>
+#include <libgnomeui/gnome-client.h>
+#include <libgnomeui/gnome-ui-init.h>
 #include <glade/glade.h>
 #include "eog-save-as-dialog-helper.h"
 #include "eog-pixbuf-util.h"
@@ -136,7 +141,7 @@ on_add_button_clicked (GtkWidget *widget, gpointer user_data)
 		dlg = eog_hig_dialog_new (GTK_WINDOW (user_data),
 					  GTK_STOCK_DIALOG_WARNING, _("Option not available."), 
 					  _("To use this function you need the libexif library. Please install"
-					    " libexif (http://libexif.sf.net) and recompile Eye of Gnome."), TRUE);
+					    " libexif (http://libexif.sf.net) and recompile Eye of GNOME."), TRUE);
 		gtk_dialog_add_button (GTK_DIALOG (dlg), GTK_STOCK_OK, GTK_RESPONSE_OK);
 		gtk_widget_show_all (dlg);
 		gtk_dialog_run (GTK_DIALOG (dlg));
