@@ -1,3 +1,7 @@
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <math.h>
 #include <libgnome/gnome-macros.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
@@ -18,7 +22,7 @@
 
 /* Maximum zoom factor */
 #define MAX_ZOOM_FACTOR 20
-#define MIN_ZOOM_FACTOR 0.01
+#define MIN_ZOOM_FACTOR 0.02
 
 #define CHECK_MEDIUM 8
 #define CHECK_BLACK 0x00000000
@@ -55,9 +59,6 @@ enum {
 };
 static gint view_signals [SIGNAL_LAST];
 
-/* See image_cb_ids below; number of EogImage's signals to which we connect */
-#define NUM_IMAGE_CALLBACKS 5
-
 /* Private part of the EogScrollView structure */
 struct _EogScrollViewPrivate {
 	/* some widgets we rely on */
@@ -69,7 +70,7 @@ struct _EogScrollViewPrivate {
 
 	/* actual image */
 	EogImage *image;
-	gulong image_cb_ids [NUM_IMAGE_CALLBACKS];
+	guint image_changed_id;
 	GdkPixbuf *pixbuf;
 
 	/* zoom mode, either ZOOM_MODE_FIT or ZOOM_MODE_FREE */
@@ -131,24 +132,16 @@ static void
 free_image_resources (EogScrollView *view)
 {
 	EogScrollViewPrivate *priv;
-	int i;
 
 	priv = view->priv;
 
-	for (i = 0; i < NUM_IMAGE_CALLBACKS; i++) {
-		if (priv->image_cb_ids[i] != 0 ) {
-			g_signal_handler_disconnect (G_OBJECT (priv->image), priv->image_cb_ids[i]);
-			priv->image_cb_ids[i] = 0;
-		}
+	if (priv->image_changed_id > 0) {
+		g_signal_handler_disconnect (G_OBJECT (priv->image), priv->image_changed_id);
+		priv->image_changed_id = 0;
 	}
 
 	if (priv->image != NULL) {
-		/* FIXME: calling cancel_load() here should not be necessary;
-		 * the EogImage should stop loading if its refcount goes down to
-		 * zero.
-		 */
-		eog_image_cancel_load (priv->image);
-		g_object_unref (priv->image);
+		eog_image_data_unref (priv->image);
 		priv->image = NULL;
 	}
 
@@ -1009,6 +1002,11 @@ set_zoom_fit (EogScrollView *view)
 				   gdk_pixbuf_get_height (priv->pixbuf),
 				   priv->upscale);
 
+	if (new_zoom > MAX_ZOOM_FACTOR)
+		new_zoom = MAX_ZOOM_FACTOR;
+	else if (new_zoom < MIN_ZOOM_FACTOR)
+		new_zoom = MIN_ZOOM_FACTOR;
+
 	if (DOUBLE_EQUAL (new_zoom, priv->zoom))
 		return;
 
@@ -1710,6 +1708,22 @@ eog_scroll_view_get_zoom (EogScrollView *view)
 	return view->priv->zoom;
 }
 
+gboolean
+eog_scroll_view_get_zoom_is_min (EogScrollView *view)
+{
+	g_return_val_if_fail (EOG_IS_SCROLL_VIEW (view), FALSE);
+
+	return DOUBLE_EQUAL (view->priv->zoom, MIN_ZOOM_FACTOR);
+}
+
+gboolean
+eog_scroll_view_get_zoom_is_max (EogScrollView *view)
+{
+	g_return_val_if_fail (EOG_IS_SCROLL_VIEW (view), FALSE);
+
+	return DOUBLE_EQUAL (view->priv->zoom, MAX_ZOOM_FACTOR);
+}
+
 void
 eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 {
@@ -1723,10 +1737,6 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 		return;
 	}
 
-	if (image != NULL) {
-		g_object_ref (image);
-	}
-
 	if (priv->image != NULL) {
 		free_image_resources (view);
 		if (GTK_WIDGET_DRAWABLE (priv->display) && image == NULL) {
@@ -1736,24 +1746,31 @@ eog_scroll_view_set_image (EogScrollView *view, EogImage *image)
 	g_assert (priv->image == NULL);
 	g_assert (priv->pixbuf == NULL);
 
-
 	priv->progressive_state = PROGRESSIVE_NONE;
 	if (image != NULL) {
-		priv->image = image;
+		eog_image_data_ref (image);
 
-		priv->image_cb_ids[0] = g_signal_connect (priv->image, "loading_update",
-							  (GCallback) image_loading_update_cb, view);
-		priv->image_cb_ids[1] = g_signal_connect (priv->image, "loading_finished",
-							  (GCallback) image_loading_finished_cb, view);
-		priv->image_cb_ids[2] = g_signal_connect (priv->image, "loading_failed",
-							  (GCallback) image_loading_failed_cb, view);
-		priv->image_cb_ids[3] = g_signal_connect (priv->image, "loading_cancelled",
-							  (GCallback) image_loading_cancelled_cb, view);
-		priv->image_cb_ids[4] = g_signal_connect (priv->image, "image_changed",
-							  (GCallback) image_changed_cb, view);
+		if (priv->pixbuf == NULL) {
+			priv->pixbuf = eog_image_get_pixbuf (image);
+			priv->progressive_state = PROGRESSIVE_NONE;
+			set_zoom_fit (view);
+			check_scrollbar_visibility (view, NULL);
+			gtk_widget_queue_draw (GTK_WIDGET (priv->display));
 
-		eog_image_load (priv->image, EOG_IMAGE_LOAD_DEFAULT);
+		}
+		else if (priv->interp_type != GDK_INTERP_NEAREST &&
+			 !is_unity_zoom (view))
+		{
+			/* paint antialiased image version */
+			priv->progressive_state = PROGRESSIVE_POLISHING;
+			gtk_widget_queue_draw (GTK_WIDGET (priv->display));
+		}
+
+		priv->image_changed_id = g_signal_connect (image, "image_changed",
+							   (GCallback) image_changed_cb, view);
 	}
+	
+	priv->image = image;
 }
 
 void
