@@ -1,13 +1,22 @@
+#include <config.h>
 #include <gtk/gtk.h>
 #include <glade/glade.h>
-#include <libgnomeui/libgnomeui.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
+#include "util.h"
 #include "eog-preferences.h"
 #include "eog-config-keys.h"
 #include "eog-hig-dialog.h"
 
+#ifdef G_OS_WIN32
+
+#undef EOG_DATADIR
+#define EOG_DATADIR eog_get_datadir ()
+
+#endif
+
 #define GCONF_OBJECT_KEY             "GCONF_KEY"
 #define GCONF_OBJECT_VALUE           "GCONF_VALUE"
+#define OBJECT_WIDGET                "OBJECT_WIDGET"
 
 static void
 check_toggle_cb (GtkWidget *widget, gpointer data)
@@ -23,6 +32,28 @@ check_toggle_cb (GtkWidget *widget, gpointer data)
 			       NULL);
 }
 
+static void
+check_auto_advance_toggle_cb (GtkWidget *widget, gpointer data)
+{
+	char *key = NULL;
+	GtkWidget *child = NULL;
+	gboolean state = FALSE;
+
+	key = g_object_get_data (G_OBJECT (widget), GCONF_OBJECT_KEY);
+	if (key == NULL) return;
+
+	state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
+	
+	gconf_client_set_bool (GCONF_CLIENT (data),
+			       key, state,			       
+			       NULL);
+	
+	child = g_object_get_data (G_OBJECT (widget), OBJECT_WIDGET);
+	if (child != NULL) {
+		g_object_set (child, "sensitive", state, NULL);
+	}
+}
+
 static void
 spin_button_changed_cb (GtkWidget *widget, gpointer data)
 {
@@ -38,22 +69,20 @@ spin_button_changed_cb (GtkWidget *widget, gpointer data)
 }
 
 static void
-color_change_cb (GtkWidget *widget, guint red, guint green, guint blue, guint a, gpointer data)
+color_change_cb (GtkColorButton *button, gpointer data)
 {
+	GdkColor color;
 	char *key = NULL;
 	char *value = NULL;
-	char *ptr;
-	
-	value = g_strdup_printf ("#%2X%2X%2X",
-			     red / 256,
-			     green / 256,
-			     blue / 256);
-  
-	for (ptr = value; *ptr; ptr++)
-		if (*ptr == ' ')
-			*ptr = '0';
 
-	key = g_object_get_data (G_OBJECT (widget), GCONF_OBJECT_KEY);
+	gtk_color_button_get_color (button, &color);
+
+	value = g_strdup_printf ("#%2X%2X%2X",
+				 color.red / 256,
+				 color.green / 256,
+				 color.blue / 256);
+
+	key = g_object_get_data (G_OBJECT (button), GCONF_OBJECT_KEY);
 	if (key == NULL || value == NULL) 
 		return;
 
@@ -85,53 +114,55 @@ radio_toggle_cb (GtkWidget *widget, gpointer data)
 }
 
 static void
-help_cb (GtkWidget *widget, gpointer data)
+dialog_response_cb (GtkDialog *dlg, gint res_id, gpointer data)
 {
 	GError *error = NULL;
 
-	gnome_help_display ("eog.xml", "eog-prefs", &error);
+	switch (res_id) {
+	case GTK_RESPONSE_HELP:
+		gnome_help_display ("eog.xml", "eog-prefs", &error);
 
-	if (error) {
-		GtkWidget *dialog;
+		if (error) {
+			GtkWidget *dialog;
 
-		dialog = eog_hig_dialog_new (NULL, GTK_STOCK_DIALOG_ERROR,
-					     _("Could not display help for Eye of GNOME"),
-					     error->message, TRUE);
+			dialog = eog_hig_dialog_new (NULL, GTK_STOCK_DIALOG_ERROR,
+						     _("Could not display help for Eye of GNOME"),
+						     error->message, TRUE);
 
-		gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
+			gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_OK);
 
-		g_signal_connect_swapped (dialog, "response",
-					  G_CALLBACK (gtk_widget_destroy),
-					  dialog);
+			g_signal_connect_swapped (dialog, "response",
+						  G_CALLBACK (gtk_widget_destroy),
+						  dialog);
 
-		gtk_widget_show (dialog);
-		g_error_free (error);
+			gtk_widget_show (dialog);
+			g_error_free (error);
+		}
+		break;
+
+	default:
+		gtk_widget_destroy (GTK_WIDGET (dlg));
 	}
 }
 
 void
-eog_preferences_show (GConfClient *client)
+eog_preferences_show (GtkWindow *parent, GConfClient *client)
 {
 	GtkWidget *dlg;
 	GladeXML  *xml;
 	GtkWidget *widget;
-	char *value;
+	char *value, *filename;
 	GdkColor color;
 
-	xml = glade_xml_new (DATADIR "/eog/glade/eog.glade", "Hig Preferences Dialog", "eog");
+	filename = g_build_filename (EOG_DATADIR, "eog/glade/eog.glade", NULL);
+	xml = glade_xml_new (filename, "Hig Preferences Dialog", "eog");
+	g_free (filename);
 	g_assert (xml != NULL);
 
 	dlg = glade_xml_get_widget (xml, "Hig Preferences Dialog");
 
-	widget = glade_xml_get_widget (xml, "close_button");
-	g_signal_connect_swapped (G_OBJECT (widget), "clicked", 
-				  G_CALLBACK (gtk_widget_destroy), dlg);
-
-	widget = glade_xml_get_widget (xml, "help_button");
-
-	if (widget)
-		g_signal_connect (G_OBJECT (widget), "clicked", 
-				  G_CALLBACK (help_cb), dlg);
+	g_signal_connect (G_OBJECT (dlg), "response",
+			  G_CALLBACK (dialog_response_cb), dlg);
 
 	/* interpolate flag */
 	widget = glade_xml_get_widget (xml, "interpolate_check");
@@ -181,14 +212,11 @@ eog_preferences_show (GConfClient *client)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
 
 	/* color picker */
+	g_free (value);
 	value = gconf_client_get_string (client, EOG_CONF_VIEW_TRANS_COLOR, NULL);
-	widget = glade_xml_get_widget (xml, "colorpicker");
+	widget = glade_xml_get_widget (xml, "colorbutton");
 	if (gdk_color_parse (value, &color)) {
-		gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (widget),
-					    color.red,
-					    color.green,
-					    color.blue,
-					    255);
+		gtk_color_button_set_color (GTK_COLOR_BUTTON (widget), &color);
 	}
 	g_object_set_data (G_OBJECT (widget), GCONF_OBJECT_KEY, EOG_CONF_VIEW_TRANS_COLOR);
 	g_signal_connect (G_OBJECT (widget),
@@ -215,6 +243,15 @@ eog_preferences_show (GConfClient *client)
 			  G_CALLBACK (check_toggle_cb), 
 			  client);
 
+	widget = glade_xml_get_widget (xml, "auto_advance_check");	
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
+				      gconf_client_get_bool (client, EOG_CONF_FULLSCREEN_AUTO_ADVANCE, NULL));
+	g_object_set_data (G_OBJECT (widget), OBJECT_WIDGET, glade_xml_get_widget (xml, "seconds_hbox"));
+	g_object_set_data (G_OBJECT (widget), GCONF_OBJECT_KEY, EOG_CONF_FULLSCREEN_AUTO_ADVANCE);
+	g_signal_connect (G_OBJECT (widget), 
+			  "toggled", 
+			  G_CALLBACK (check_auto_advance_toggle_cb), 
+			  client);
 
 	widget = glade_xml_get_widget (xml, "seconds_spin");
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (widget), 
@@ -224,5 +261,12 @@ eog_preferences_show (GConfClient *client)
 			  "changed", 
 			  G_CALLBACK (spin_button_changed_cb), 
 			  client);
-}
+			  
+	if (!gconf_client_get_bool (client, EOG_CONF_FULLSCREEN_AUTO_ADVANCE, NULL)) {
+		widget = glade_xml_get_widget (xml, "seconds_hbox");
+		g_object_set (widget, "sensitive", FALSE, NULL);
+	}
 
+	gtk_window_set_transient_for (GTK_WINDOW (dlg), parent);
+	gtk_widget_show_all (dlg);
+}
